@@ -6,7 +6,15 @@
 MAN_D=/usr/local/man/man8
 
 if [ "$INIT_D" == "" ] ; then
-	if [ -d /etc/rc.d/init.d ] ; then
+	# Try non-link directories first, then try links. This helps if one of
+	# the locations is linked to another, which is the case on some distros.
+	if [ -d /etc/rc.d/init.d -a ! -L /etc/rc.d/init.d ] ; then
+		INIT_D=/etc/rc.d/init.d
+	elif [ -d /etc/rc.d -a ! -L /etc/rc.d -a ! -d /etc/rc.d/init.d ] ; then
+		INIT_D=/etc/rc.d
+	elif [ -d /etc/init.d -a ! -L /etc/init.d ] ; then
+		INIT_D=/etc/init.d
+	elif [ -d /etc/rc.d/init.d ] ; then
 		INIT_D=/etc/rc.d/init.d
 	elif [ -d /etc/rc.d ] ; then
 		INIT_D=/etc/rc.d
@@ -18,6 +26,20 @@ if [ "$INIT_D" == "" ] ; then
 	fi
 fi
 
+if ( which invoke-rc.d > /dev/null ) ; then
+	# Debian uses invoke-rc.d
+	RCPROG=invoke-rc.d
+	INITSCRIPT=laptop-mode
+elif ( which service > /dev/null ) ; then
+	# RedHat uses service
+	RCPROG=service
+	INITSCRIPT=laptop-mode
+else
+	# Any other -- we start it ourselves.
+	RCPROG=
+	INITSCRIPT=$INIT_D/laptop-mode
+fi
+
 if [ "`whoami`" != "root" ] ; then
 	echo "You need to be root to install the laptop mode tools."
 	exit 10
@@ -27,6 +49,9 @@ if [ ! -f /proc/sys/vm/laptop_mode ] ; then
 	echo "Warning: the kernel you are running does not support laptop mode."
 fi
 
+echo 'Stopping existing laptop mode service (if any).'
+$RCPROG $INITSCRIPT stop
+
 INSTALL="install -o root -g root"
 
 mkdir -p /etc/laptop-mode /etc/laptop-mode/batt-start /etc/laptop-mode/batt-stop /etc/laptop-mode/lm-ac-start /etc/laptop-mode/lm-ac-stop /etc/laptop-mode/nolm-ac-start /etc/laptop-mode/nolm-ac-stop
@@ -41,6 +66,13 @@ elif ( ! $INSTALL -m 600 etc/laptop-mode/laptop-mode.conf /etc/laptop-mode ) ; t
 	exit 12
 fi
 
+mkdir -p /etc/laptop-mode/nolm-ac-start
+mkdir -p /etc/laptop-mode/nolm-ac-stop
+mkdir -p /etc/laptop-mode/lm-ac-start
+mkdir -p /etc/laptop-mode/lm-ac-stop
+mkdir -p /etc/laptop-mode/batt-start
+mkdir -p /etc/laptop-mode/batt-stop
+
 if [ -f /etc/default/laptop-mode -a "$ALREADY_EXISTED" -eq 0 ] ; then
         echo "Found old configuration file in /etc/default/laptop-mode. Moving to the new location."
 	if ( ! mv /etc/default/laptop-mode /etc/laptop-mode/laptop-mode.conf ) ; then
@@ -188,20 +220,6 @@ if ( ! cp man/* $MAN_D ) ; then
   exit 23
 fi
 
-if ( which invoke-rc.d > /dev/null ) ; then
-	# Debian uses invoke-rc.d
-	RCPROG=invoke-rc.d
-	INITSCRIPT=laptop-mode
-elif ( which service > /dev/null ) ; then
-	# RedHat uses service
-	RCPROG=service
-	INITSCRIPT=laptop-mode
-else
-	# Any other -- we start it ourselves.
-	RCPROG=
-	INITSCRIPT=$INIT_D/laptop-mode
-fi
-
 if ( ! $RCPROG $INITSCRIPT start ) ; then
 	echo "$0: Could not start laptop mode init script /etc/init.d/laptop-mode."
 	exit 24
